/*==========================================================================*\
|                                                                            |
|                                                                            |
| PC-side Bootstrap Loader communication Application                         |
|                                                                            |
| See main.c for full version and legal information                          |
|                                                                            |
\*==========================================================================*/

#include <windows.h>
#include <string.h>
#include <stdio.h>
#include <fcntl.h>
#include <conio.h>
#include "hiddevice.h"
#include "BSL_IO_USB.h"

int USB_verbose = 0; /* verbose mode flag, default off */
struct strHidDevice device; /*HID device to use */

/*******************************************************************************
*Function:    USB_initializeCommPort
*Description: initializes a COM port for communication
*Parameters:
*             non
*Returns:
*             none
*******************************************************************************/
void USB_initializeCommPort()
{
	HID_Init(&device);
	HID_Open(&device, 0x2047, 0x0200, 0);
}

/*******************************************************************************
*Function:    USB_setVerbose
*Description: sets Verbose mode to on or off for debugging
*Parameters: 
*             unsigned int verb   1 == verbose mode on, 0 == verbose mode off
*Returns:
*             none
*******************************************************************************/
void USB_setVerbose(unsigned int verb)
{
	USB_verbose = verb;
}

/*******************************************************************************
*Function:     USB_initialize_BSL
*Description:  initializes the BSL communication
*Parameters:
*              unsigned char* comPort   COM-port where BSL is attached
*Returns:
*              none
*******************************************************************************/
unsigned char USB_initialize_BSL(unsigned char* comPort, long rate, unsigned int mode, unsigned char family)
{
	USB_initializeCommPort();
	return BSL_ACK;
}

/*******************************************************************************
*Function:     USB_sendData
*Description:  sends data via USB communication
*Parameters:
*              unsigned char *buf           buffer with data
*              unsigned int   num_bytes     indicates the number of bytes that will be transmitted
*Returns:
*              none
*******************************************************************************/
void USB_sendData(unsigned char* buf, unsigned int num_bytes)
{
	unsigned int i;
	unsigned char USB_buf[MAX_USB_FRAME_SIZE];
	memset(USB_buf, 0xAC, MAX_USB_FRAME_SIZE);

	USB_buf[0]= (char)num_bytes;
	num_bytes++;
	for(i = 1; i < num_bytes; i++)
	{
		USB_buf[i] = buf[i-1];
	}
	if (USB_verbose)
	{
		int i;
		for(i = 0; i< MAX_USB_FRAME_SIZE; i++)
		{
			printf( "{%2.2x} ", (USB_buf[i] & 0xFF));
		}
	}
	HID_WriteFile(&device, USB_buf, num_bytes + 1);
	return;
}

/*******************************************************************************
*Function:    USB_recieveData
*Description: recieves data via USB communication
*Parameters:
*             unsigned char * buf       pointer to buffer where data will be stored
*Returns:
*             int                       number of recieved data bytes
*******************************************************************************/
int USB_receiveData(unsigned char* buf)
{
	int retVal = 0;
	char temp_buf[MAX_USB_FRAME_SIZE+1];
	if (HID_ReadFile( &device, temp_buf) == HID_DEVICE_SUCCESS)
	{
		int i;
		if (USB_verbose)
		{
			printf( "\n--------------------------------\n");
		}
		retVal = temp_buf[1];
		for (i = 0; i <= retVal; i++)
		{
			buf[i] = temp_buf[i+2];
		}
		if (USB_verbose)
		{
			for (i = 0; i< sizeof(temp_buf); i++)
			{
				printf("[%2.2x] ", (temp_buf[i] & 0xFF));
			}
		}
	}
	return retVal;
}

/*******************************************************************************
*Function:     USB_dalay
*Description:  dalay function
*Parameters:
*              int time       time to wait
*Returns:
*              void
*******************************************************************************/
void USB_delay(int time) /* exported! */
{ 
#ifndef WIN32
	DWORD startTime= GetTickCount();
	while (UART_calcTimeout(startTime) < time);
#else
	Sleep(time);
#endif
}